/*
 * Copyright 1999-2018 Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.druid.bvt.sql.postgresql.select;

import java.util.List;

import org.junit.Assert;

import com.alibaba.druid.sql.PGTest;
import com.alibaba.druid.sql.SQLUtils;
import com.alibaba.druid.sql.ast.SQLStatement;
import com.alibaba.druid.sql.dialect.postgresql.parser.PGSQLStatementParser;
import com.alibaba.druid.sql.dialect.postgresql.visitor.PGSchemaStatVisitor;

public class PGSelectTest18 extends PGTest {

    public void test_0() throws Exception {
        String sql = "WITH RECURSIVE t(n) AS ("
                + "    SELECT 1"
                + "  UNION ALL"
                + "    SELECT n+1 FROM t"
                + ")"
                + "SELECT n FROM t LIMIT 100;";

        PGSQLStatementParser parser = new PGSQLStatementParser(sql);
        List<SQLStatement> statementList = parser.parseStatementList();
        SQLStatement stmt = statementList.get(0);
        {
            String result = SQLUtils.toPGString(stmt);
            Assert.assertEquals("WITH RECURSIVE t (n) AS (\n" +
                    "\t\tSELECT 1\n" +
                    "\t\tUNION ALL\n" +
                    "\t\tSELECT n + 1\n" +
                    "\t\tFROM t\n" +
                    "\t)\n" +
                    "SELECT n\n" +
                    "FROM t\n" +
                    "LIMIT 100;", result);
        }
        {
            String result = SQLUtils.toPGString(stmt, SQLUtils.DEFAULT_LCASE_FORMAT_OPTION);
            Assert.assertEquals("with recursive t (n) as (\n" +
                    "\t\tselect 1\n" +
                    "\t\tunion all\n" +
                    "\t\tselect n + 1\n" +
                    "\t\tfrom t\n" +
                    "\t)\n" +
                    "select n\n" +
                    "from t\n" +
                    "limit 100;", result);
        }
        
        Assert.assertEquals(1, statementList.size());

        PGSchemaStatVisitor visitor = new PGSchemaStatVisitor();
        stmt.accept(visitor);

        System.out.println("Tables : " + visitor.getTables());
        System.out.println("fields : " + visitor.getColumns());
//        System.out.println("coditions : " + visitor.getConditions());

        Assert.assertEquals(0, visitor.getColumns().size());
        Assert.assertEquals(0, visitor.getTables().size());
    }
}

// select categoryId , offerIds from cnres.function_select_get_spt_p4p_offer_list (' 1031918 , 1031919 , 1037004 ') as
// a(categoryId numeric,offerIds character varying(4000))
// select memberId , offerIds from cnres.function_select_get_seller_hot_offer_list('\'gzyyd168\'') as a(memberId
// character varying(20),offerIds character varying(4000))

